/* GETGUI - get user responses for shell scripts, using a simple GUI */

#define BUTTONS 'b'	/* present user with one or more buttons.. wait for user to click on one.. report the selected button. */
#define FILEPICK 'f'	/* present a select box with files, allow user to pick one.. report the selected file or "../" or "" */
#define INPROGRESS 'p'	/* present text with optional buttons.. asyncronous.. communicates via 2 files */
#define KEYIN 'k'	/* present a textfield, allow user to type in a response.. report the response */
#define PULLDOWN 'd'	/* present a pulldown, allow user to select a response.. report the response */
#define SELECTBOX 's'	/* present a select box, allow user to select an entire row .. report the response */

/* pickreps */
#define ENTIRE_TEXT 0	/* return the entire text */
#define FIRST_TOKEN 1	/* return only the first whitespace-delimited token - available w/ pulldowns & selectboxes */
#define INDEX_NUM 2	/* return the position # of the selection, first=1 - available w/ buttons & selectboxes */


#include "plg.h"

int omitnewline = 0;
int rtncode = 0;

main( argc, argv )
int argc;
char **argv;
{
int i, j;
char buf[512];
int prefab, updir;
char *prompt, *content;
int ns1, b1, tx1, b2, k1, p1;  /* GUI object ids */
char filepat[256];
char *winname, *buttoncolor;
char *buttons;
int allowcomma;
int fieldlen;
char *initval;
int pwmode;
int wordselect, chunkselect;
int multichoice;
char *okstr;
char *clrfile, *buttonfile;
int lastobj, stopobj;
char *dir;
int pickrep; /* see #defines above */
int buttonwrap;
int promptsize;
char *promptalign;
char *promptcolor;
int buttonsize;
char *buttonalign;
int hrule;
char *title;
int horiz;
int echoargs;
char *pullorient;
int keymax;
int sleepmin;
int keyend, keycursor;

if( argc < 2 || strcmp( argv[1], "-help" )==0 || strcmp( argv[1], "--help" )==0 ) {
	fprintf( stderr, "getgui 2.02 (c) 2004-5 Steve Grubb (GPL, see http://getgui.sourceforge.net)\n" );
	fprintf( stderr, "Usage: getgui  messagetext  [-buttons a,b,c]   (etc)\n" );
	fprintf( stderr, "Examples:\n" );
	fprintf( stderr, "  getgui \"The file has been renamed.\"  \n" );
	fprintf( stderr, "  getgui \"Select a file:\"  -filepick -updir \n" );
	fprintf( stderr, "  getgui \"Do you want to continue?\"  -buttons \"Yes|No\" \n" );
	fprintf( stderr, "  getgui \"Enter your serial number:\"  -keyin 12 \n" );
	fprintf( stderr, "  getgui \"Enter password:\"  -password 10 \n" );
	fprintf( stderr, "  getgui \"Installation in progress..\"  -progress  -clrfile tmpclr \n" );
	fprintf( stderr, "  getgui \"In progress..\"  -progress  -buttons Cancel  -clrfile tmpclr  -buttonfile tmpbtn \n" );
	exit( 0 );
	}

buttons = NULL;
prompt = content = NULL;
winname = NULL;
buttoncolor = "yellow";
strcpy( filepat, "" );
fieldlen = 15;
initval = NULL;
pwmode = 0;
wordselect = chunkselect = 0;
multichoice = 0;
okstr = "OK";
updir = 0;
clrfile = buttonfile = NULL;
prefab = BUTTONS;
getgui_set( "listsep", 0, 0.0, "\n" );
dir = NULL;
pickrep = ENTIRE_TEXT;
buttonwrap = 60;
promptsize = 10;
promptalign = buttonalign = "center";
promptcolor = "black";
buttoncolor = "";
buttonsize = 12;
hrule = 0;
title = NULL;
horiz = 0;
echoargs = 0;
pullorient = NULL;
keymax = 0;
sleepmin = 0;
keyend = 0;
keycursor = 999;


for( i = 1; i < argc; i++ ) {
	if( i == 1 && argv[i][0] != '-' ) prompt = argv[i];
	else if( strcmp( argv[i], "-title" )==0 && i < argc-1 ) { title = argv[i+1]; i++; }
	else if( strcmp( argv[i], "-horiz" )==0 ) {  /* horizontal orientation, a la wizard mode */
		horiz = 1; 
		promptalign = "left"; 
		buttonalign = "right"; 
		}
	else if( strcmp( argv[i], "-buttons" )==0 && i < argc-1 ) { buttons = argv[i+1]; i++; }
	else if( strcmp( argv[i], "-filepick" )==0 ) {
		prefab = FILEPICK; 
		if( i < argc-1 && argv[i+1][0] != '-' ) { sprintf( filepat, "!ls -C %s", argv[i+1] ); content = filepat; i++; }
		}
	else if( strcmp( argv[i], "-progress" )==0 ) prefab = INPROGRESS; 
	else if( strcmp( argv[i], "-keyin" )==0 || strcmp( argv[i], "-password" )==0 ) {
		prefab = KEYIN;
		if( argv[i][1] == 'p' ) pwmode = 1;
		if( i < argc-1 && argv[i+1][0] != '-' ) { fieldlen = atoi( argv[i+1] ); i++; }
		}
	else if( strcmp( argv[i], "-keymax" )==0 && i < argc-1 ) { keymax = atoi( argv[i+1] ); i++; }
	else if( strcmp( argv[i], "-keycursor" )==0 && i < argc-1 ) { keycursor = atoi( argv[i+1] ); i++; }
	else if( strcmp( argv[i], "-pulldown" )==0 && i < argc-1 ) { prefab = PULLDOWN; content = argv[i+1]; i++; }
	else if( strcmp( argv[i], "-selectbox" )==0 && i < argc-1 ) { prefab = SELECTBOX; content = argv[i+1]; i++; }
	else if( strcmp( argv[i], "-msg" )==0 && i < argc-1 ) { prompt = argv[i+1]; i++; }
	else if( strcmp( argv[i], "-initval" )==0 && i < argc-1 ) { initval = argv[i+1]; i++; }
	else if( strcmp( argv[i], "-okstr" )==0 && i < argc-1 ) { okstr = argv[i+1]; i++; }
	else if( strcmp( argv[i], "-clrfile" )==0 && i < argc-1 ) { clrfile = argv[i+1 ]; getgui_set( "clrfile", 0, 0.0, clrfile ); i++; }
	else if( strcmp( argv[i], "-ipick" )==0 ) pickrep = INDEX_NUM;
	else if( strcmp( argv[i], "-shortpick" )==0 ) pickrep = FIRST_TOKEN;
	else if( strcmp( argv[i], "-msgsize" )==0 && i < argc-1 ) { promptsize = atoi( argv[i+1] ); i++; }
	else if( strcmp( argv[i], "-msgalign" )==0 && i < argc-1 ) { promptalign = argv[i+1]; i++; }
	else if( strcmp( argv[i], "-msgcolor" )==0 && i < argc-1 ) { promptcolor = argv[i+1]; i++; }
	else if( strcmp( argv[i], "-updir" )==0 ) updir = 1;
	else if( strcmp( argv[i], "-dir" )==0 && i < argc-1 ) { dir = argv[i+1]; i++; }
	else if( strcmp( argv[i], "-btnsize" )==0 && i < argc-1 ) { buttonsize = atoi( argv[i+1] ); i++; }
	else if( strcmp( argv[i], "-btncolor" )==0 && i < argc-1 ) { buttoncolor = argv[i+1]; i++; }
	else if( strcmp( argv[i], "-btnalign" )==0 && i < argc-1 ) { buttonalign = argv[i+1]; i++; }
	else if( strcmp( argv[i], "-btnfile" )==0 && i < argc-1 ) { buttonfile = argv[i+1]; i++; }
	else if( strcmp( argv[i], "-btnwrap" )==0 && i < argc-1 ) { buttonwrap = atoi( argv[i+1] ); i++; }
	else if( strncmp( argv[i], "-multi", 6 )==0 ) multichoice = 1;
	else if( strcmp( argv[i], "-wordselect" )==0 ) wordselect = 1;
	else if( strcmp( argv[i], "-chunkselect" )==0 ) chunkselect = 1;
	else if( strcmp( argv[i], "-pullorient" )==0 && i < argc-1 ) { pullorient = argv[i+1]; i++; }
	else if( strcmp( argv[i], "-content" )==0 && i < argc-1 ) { content = argv[i+1]; i++; }
	else if( strcmp( argv[i], "-omitn" )==0 ) omitnewline = 1;
	else if( strcmp( argv[i], "-commalist" )==0 ) getgui_set( "listsep", 0, 0.0, "," );
	else if( strcmp( argv[i], "-winname" )==0 && i < argc-1 ) { winname = argv[i+1]; i++; }
	/* integer settings */
	else if( GL_smember( argv[i], "-winx -winy -keysize -selsize" ) && i < argc-1 ) { getgui_set( &argv[i][1], atoi(argv[i+1]), 0.0, "" ); i++; }
	/* double settings */
	else if( GL_smember( argv[i], "-winmarg -btnsep -winfixw -winfixh" ) && i < argc-1 ) { getgui_set( &argv[i][1], 0, atof(argv[i+1]), "" ); i++; }
	/* string settings */
	else if( GL_smember( argv[i], "-bgcolor -btndress" ) && i < argc-1 ) { getgui_set( &argv[i][1], 0, 0.0, argv[i+1] ); i++; }

	else if( strcmp( argv[i], "-sleepmin" )==0 && i < argc-1 ) { sleepmin = atoi( argv[i+1] ); i++; }
	else if( strcmp( argv[i], "-keyend" )==0 ) keyend = 1;
	else if( strcmp( argv[i], "-async" )==0 ) getgui_set( "async", 1, 0.0, "" );
	else if( strcmp( argv[i], "-echoargs" )==0 ) echoargs = 1;
	else if( i > 1 ) fprintf( stderr, "getgui: %s is not a recognized option.\n", argv[i] );
	}

if( echoargs ) {
	int quotes;
	fprintf( stderr, "\n" );
	for( i = 0; i < argc; i++ ) {
		if( strcmp( argv[i], "-echoargs" )==0 ) continue;
		quotes = 0;
		for( j = 0; argv[i][j] != '\0'; j++ ) if( GL_member( argv[i][j], " ()*|^;\\`'" )) { quotes = 1; break; }
		if( quotes ) fprintf( stderr, "\"%s\" ", argv[i] );
		else fprintf( stderr, "%s ", argv[i] );
		}
	fprintf( stderr, "\n\n" );
	}


if( sleepmin != 0 ) {
	sleep( sleepmin * 60 ); 
	}



ns1 = b1 = b2 = k1 = p1 = 0;  

/* if not doing an in-progress message, or any other prefab, then at least an OK button is required. */
if( ( prefab == BUTTONS || prefab == SELECTBOX ) && buttons == NULL ) buttons = okstr; 

if( winname == NULL ) {   /* assign an appropriate generic window name */
	if( prefab == INPROGRESS  ) winname = "In progress";
	else if( prefab == KEYIN ) winname = "Enter a response";
	else if( prefab == BUTTONS && stricmp( buttons, okstr )==0 ) winname = "Info";
	else winname = "Select";
	}


if( dir != NULL ) chdir( dir );   /* if a working directory was specified, chdir to it now.. */


getgui_init( winname, 0 );   /* initialize things.. */



/* do the text message/prompt.... */
if( prompt == NULL ) {
	if( prefab == FILEPICK ) prompt = "Select a file:";
	else prompt = "       ";
	}
else convertnl( prompt );
tx1 = getgui_text( 4.0, 4.0, prompt, promptalign, "top", promptsize, promptcolor, 0 );
lastobj = tx1;

/* do the title */
if( title != NULL ) {
	convertnl( title );
	getgui_relalign_next( "top", tx1, 0.2, (horiz)?"left":"center", tx1, 0.0 );
	/* getgui_relalign_next( "top", tx1, 0.2, "", 0, 0.0 ); */
	getgui_text( 4.0, 4.0, title, (horiz)?"left":"center", "bottom", promptsize+2, "black", 1 );
	}

/* do \n conversion in content var */
if( content != NULL ) convertnl( content );


if( prefab == FILEPICK ) {	/* this prefab has its own GO module.. all others use the common one below..*/
	if( buttons == NULL ) buttons = "OK|Cancel";
	if( content == NULL ) content = "!ls -C";	/* fallback default */
	if( initval == NULL ) initval = "";
	getgui_relalign_next( "bottom", lastobj, 0.2, "center", lastobj, 0.0 );
	/* getgui_relalign_next( "bottom", lastobj, 0.2, "", 0, 0.0 ); */
	ns1 = getgui_nameselect( 4.0, 2.0, "center", "top", content, "word", initval, "single", 50, pickrep );
	lastobj = ns1;
	if( updir ) {
		getgui_relalign_next( "bottom", lastobj, 0.25, "right", ns1, 0.0 );
		b2 = getgui_buttons( 0.0, 0.0, "right", "Go up a level", 9, "gray(0.8)", 0, "single" );
		}
	else b2 = 0;
	getgui_relalign_next( "bottom", lastobj, 0.4, buttonalign, lastobj, 0.0 );
	/* getgui_relalign_next( "bottom", lastobj, 0.4, "", 0, 0.0 ); */
	b1 = getgui_buttons( 4.0, 1.0, buttonalign, buttons, buttonsize, buttoncolor, buttonwrap, "single" );
	RETRY:
	getgui_go( 0, b1, b2, 0 );
	if( updir ) {
		getgui_getval( b2, buf, ENTIRE_TEXT );/* up a level */
		if( strcmp( buf, "" ) != 0 ) { printresult( "../" ); exit( 0 ); } 
		}
	getgui_getval( b1, buf, ENTIRE_TEXT );
	if( strcmp( buf, okstr )==0 ) getgui_getval( ns1, buf, ENTIRE_TEXT );
	printresult( buf ); 
	exit( 0 );
	}


else if( prefab == KEYIN ) {
	if( initval == NULL ) initval = "";
	if( horiz ) getgui_relalign_next( "center", tx1, 0.0, "right", tx1, 0.2 );
	else getgui_relalign_next( "bottom", lastobj, 0.4, "", 0, 0.0 );
	k1 = getgui_textfield( 4.0, 1.0, (horiz)?"left":"center", fieldlen, keymax, initval, keycursor, (pwmode)?"P":"" );
	lastobj = k1;
	}


else if( prefab == PULLDOWN || prefab == SELECTBOX ) {
	if( initval == NULL ) initval = "";
	if( content == NULL ) {
		content = "(No choices available)";
		buttons = okstr;
		}
	if( pullorient == NULL ) {
		if( buttons == NULL ) pullorient = "top";
		else pullorient = "bottom";
		}
	if( horiz ) getgui_relalign_next( "center", tx1, 0.0, "right", tx1, 0.2 );
	else getgui_relalign_next( "bottom", lastobj, 0.4, "", 0, 0.0 );
	if( prefab == PULLDOWN ) p1 = getgui_pulldown( 4.0, 1.0, (horiz)?"left":"center", 
		pullorient, content, initval, pickrep );
	else p1 = getgui_nameselect( 4.0, 2.0, (horiz)?"left":"center", (horiz)?"center":"top", content, 
		(wordselect)?"word":((chunkselect)?"chunk":"line"), initval, (multichoice)?"multiple":"single", 0, pickrep );
	lastobj = p1;
	}



if( buttons != NULL ) {
	getgui_relalign_next( "bottom", lastobj, (horiz)?0.7:0.4, buttonalign, lastobj, (horiz&&prefab==BUTTONS)?1.75:0.0 ); 
	b1 = getgui_buttons( 4.0, 1.0, buttonalign, buttons, buttonsize, buttoncolor, buttonwrap, "single" );
	lastobj = b1;
	}


/* now GO.. */

if( prefab == INPROGRESS ) {
	getgui_go( 0, b1, 0, 0 );  /* if no buttons this returns upon detection of clrfile.. */
	if( buttons == NULL && clrfile == NULL ) sleep( 2 );
	else if( b1 && buttonfile != NULL ) { /* a button was pressed.. */
		FILE *fp;
		fp = fopen( buttonfile, "w" );
		if( fp == NULL ) fprintf( stderr, "getgui: cannot open buttonfile (%s)\n", buttonfile );
		else	{
			getgui_getval( b1, buf, pickrep );
			fprintf( fp, "%s\n", buf ); 
			fclose( fp );
			}
		}
	exit( 0 );
	}

else if( prefab == BUTTONS ) {
	getgui_go( 0, b1, 0, 0 );
	getgui_getval( b1, buf, pickrep );
	if( strcmp( buttons, okstr ) != 0 ) printresult( buf ); /* if only button is "OK", don't report it */
	if( pickrep == INDEX_NUM ) rtncode = atoi( buf );
	}

else if( prefab == KEYIN ) {
	if( buttons == NULL ) getgui_go( k1, k1, 0, 0 );
	else 	{
		if( keyend ) getgui_go( k1, b1, k1, 0 );
		else getgui_go( k1, b1, 0, 0 ); 
		}
	if( buttons != NULL ) {
		getgui_getval( b1, buf, pickrep );
		if( strcmp( buf, "" ) != 0 && stricmp( buf, okstr )!= 0 ) { printresult( buf ); exit( 0 ); } 
		}
	getgui_getval( k1, buf, ENTIRE_TEXT );
	printresult( buf ); 
	}

else if( prefab == PULLDOWN || prefab == SELECTBOX ) {
	if( buttons != NULL ) getgui_go( 0, b1, 0, 0 );
	else getgui_go( 0, p1, 0, 0 );
	if( buttons != NULL ) {
		getgui_getval( b1, buf, ENTIRE_TEXT );
		if( strcmp( buf, "" ) != 0 && stricmp( buf, okstr )!= 0 ) { printresult( buf ); exit( 0 ); } 
		}
	getgui_getval( p1, buf, pickrep );
	if( buttons == NULL ) usleep( 400000 ); /* add a little more delay when there are no buttons */
	printresult( buf ); 
	if( pickrep == INDEX_NUM ) rtncode = atoi( buf );
	}

exit( rtncode );
}


/* ================================= */
/* PRINTRESULT - print the result to stdout */
/* ================================= */
printresult( result )
char *result;
{
if( omitnewline ) printf( "%s", result );
else printf( "%s\n", result );
fflush( stdout );
return( 0 );
}

/* ================================= */
/* CONVERTNL - convert backslash n to \n */
/* Also handle one leading tab on a line (as might be common in shell scripts, */
/* where long multiline prompts might be given in an indented code section. */
/* NOTE: This modifies str .. seems to be safe to run on pointers to argv  (?) */
/* ================================= */
convertnl( str )
char *str;
{
int i;
for( i = 0; str[i] != '\0'; i++ ) {
	if( str[i] == '\\' && str[i+1] == 'n' ) { str[i] = ' '; str[i+1] = '\n'; }
	if( str[i] == '\n' && str[i+1] == '\t' ) { str[i] = ' '; str[i+1] = '\n'; }
	if( str[i] == '\t' ) str[i] = ' ';
	}
return( 0 );
}
